///////////////////////////////////////////////////////////////////////////////
//                                                         
// SetTest.cc - Testing the SetT template
//                                                         
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2002 by Bjoern Lemke                        
//
// IMPLEMENTATION MODULE
//
// 
// 
// Description: Testing the SetT class
//
///////////////////////////////////////////////////////////////////////////////

#pragma implementation "SetT.h"

// INCLUDES

#include "Exception.h"
#include "BigInteger.h"
#include "Chain.h"
#include "SetT.h"

// DEFINES

ostream& operator << (ostream& s, SetT<int>& x)
{

     int* pElement = x.First();
  
     s << "{";
     
     while (pElement)
     {      
	  s << *pElement ;
	  pElement = x.Next(); 
	  if (pElement)
	       s << ",";
     }
     s << "}";
     
     return s;
  
}

int main(int argc, char **argv)
{

    try {
	
	cout << "------------------------" << endl;
	cout << "Testing class SetT ..." << endl;
	cout << "   Constructors and Destructors...";
	if ( 1 )
	{
	    
	    SetT<int> mySet1;
	    SetT<Chain> mySet2;
	    SetT<BigInteger> mySet3;
	    
	}
	cout << "ok" << endl;
	
	cout << "   Member methods...";      
	
	
	SetT<int> intSet;
	SetT<Chain> chainSet;
	
	srandom(time(0));
	
	unsigned long count;
	
	for ( int i = 1 ; i < 3 ; i++ )
	{
	    intSet.Empty();
	    chainSet.Empty();
	    
	    count = 1000;      
	    while ( count > 0 )
	    {
		intSet.Insert( random() % 100 );
		chainSet.Insert( Chain(random() % 100 ) );
		count--;
	    }
	    
	    count = 1000;
	    
	    while ( count > 0 )
	    {
		intSet.Remove( random() % 100 );
		chainSet.Remove( Chain(random() % 100 ) );
		count--;	 
	    }
	    
	    
	    while ( count > 0 )
	    {
		intSet.Find( random() % 100 );
		chainSet.Find( Chain(random() % 100 ) );
		count--;	 
	    }
	    
	    int* pI = intSet.First();
	    while ( pI )
	    {
		pI = intSet.Next();
	    }
	    Chain* pS = chainSet.First();
	    while ( pS )
	    {
		pS = chainSet.Next();
	    }
	}
	
	intSet.Empty();
	chainSet.Empty();
	
	cout << "ok" << endl;
	
	cout << "   Operators...";      
	
	SetT<int> intSet2;
	intSet2 = intSet;
	
	SetT<Chain> chainSet2;
	chainSet2 = chainSet;
	
	
	SetT<int> intSet3;
	SetT<Chain> chainSet3;
	
	intSet3.Insert(9999);
	chainSet3.Insert("9999");
	
	
	intSet = intSet2 + intSet3;
	chainSet = chainSet2 + chainSet3;
	
	intSet = intSet2 - intSet3;
	chainSet = chainSet2 - chainSet3;
	
	intSet += intSet3;
	chainSet += chainSet3;
	
	intSet ^= intSet3;
	chainSet ^= chainSet3;
	
	bool b;
	b = intSet == intSet3;
	b = chainSet == chainSet3;
	
	intSet.Size();
	chainSet.Size();
	
	intSet.isEmpty();
	chainSet.isEmpty();
	
	cout << "ok" << endl;

	cout << "------------------------" << endl;	
	
    }
    catch (Exception e)
    {
	cout << "failed" << endl;
    }
}

