#ifndef _BIGDECIMAL_H_INCLUDED_
#define _BIGDECIMAL_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// BigDecimal.h
// ------------
// BigDecimal class definition
//                                                         
// Design and Implementation by Bjoern Lemke               
// 
// (C)opyright 2000-2007 Bjoern Lemke                        
//                                                         
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// INTERFACE MODULE
//
// Class: BigDecimal
// 
// Description: BigDecimal class arbitrary decimal values
//
// Status: IMPLEMENTED 
//
///////////////////////////////////////////////////////////////////////////////


// INCLUDES
#include "Chain.h"

class BigDecimal {

public:
    
    BigDecimal();

    BigDecimal(const Chain& s);
    ~BigDecimal();
    BigDecimal(int dim, int scale);
    BigDecimal(const Chain& val, int scale);


    BigDecimal abs() const;
    bool isPositive() const;
    void negate();

    BigDecimal add(const BigDecimal& d) const;
    BigDecimal sub(const BigDecimal& d) const;
    BigDecimal mul(const BigDecimal& d) const;
    BigDecimal div(const BigDecimal& d) const;

    int getScale() const;
    int length() const;
    
    Chain toChain() const;
    
    BigDecimal& operator = ( const BigDecimal& d);
    BigDecimal& operator += ( const BigDecimal& d);
    BigDecimal& operator -= ( const BigDecimal& d);
    bool operator == ( const BigDecimal& d) const;
    bool operator != ( const BigDecimal& d) const;
    bool operator < ( const BigDecimal& d) const;
    bool operator > ( const BigDecimal& d) const;
    bool operator <= ( const BigDecimal& d) const;
    bool operator >= ( const BigDecimal& d) const;

    friend BigDecimal operator + ( const BigDecimal& d1, const BigDecimal& d2);
    friend BigDecimal operator - ( const BigDecimal& d1, const BigDecimal& d2);
    friend BigDecimal operator * ( const BigDecimal& d1, const BigDecimal& d2);
    friend BigDecimal operator / ( const BigDecimal& d1, const BigDecimal& d2);
    friend ostream& operator << (ostream& s, const  BigDecimal& d);
    
private:

    Chain mulDec(const Chain& val, int dim) const;

    bool _isPositive;
    Chain _val;
    int _dim;
    int _scale;

};



#endif



