#ifndef _BIGINTEGER_H_INCLUDED_
#define _BIGINTEGER_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// BigInteger.h
// ------------
// BigInteger class definition
//                                                         
// Design and Implementation by Bjoern Lemke               
// 
// (C)opyright 2000-2005 Bjoern Lemke                        
//                                                         
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// INTERFACE MODULE
//
// Class: BigInteger 
// 
// Description: BigInteger class arbitrary decimal values
//
// Status: IMPLEMENTED 
//
///////////////////////////////////////////////////////////////////////////////


// INCLUDES
#include "Chain.h"

class BigInteger {

public:
    
    BigInteger();
    BigInteger(const BigInteger& bi);
    BigInteger(const Chain& s);
    ~BigInteger();

    BigInteger abs() const;
    bool isPositive() const;
    void negate();

    BigInteger add(const BigInteger& d) const;
    BigInteger sub(const BigInteger& d) const;
    BigInteger mul(const BigInteger& d) const;
    BigInteger div(const BigInteger& d) const;

    int length() const;
    int getDigit(int i) const;

    
    Chain toChain() const;
    
    BigInteger& operator = ( const BigInteger& d);
    BigInteger& operator += ( const BigInteger& d);
    BigInteger& operator -= ( const BigInteger& d);
    bool operator == ( const BigInteger& d) const;
    bool operator != ( const BigInteger& d) const;
    bool operator < ( const BigInteger& d) const;
    bool operator > ( const BigInteger& d) const;
    bool operator <= ( const BigInteger& d) const;
    bool operator >= ( const BigInteger& d) const;

    friend BigInteger operator + ( const BigInteger& d1, const BigInteger& d2);
    friend BigInteger operator - ( const BigInteger& d1, const BigInteger& d2);
    friend BigInteger operator * ( const BigInteger& d1, const BigInteger& d2);
    friend BigInteger operator / ( const BigInteger& d1, const BigInteger& d2);

    friend ostream& operator << (ostream& s, const  BigInteger& d);


    
private:

    BigInteger mulDigit(int f) const;

    bool _isPositive;
    Chain _val;

};



#endif



