///////////////////////////////////////////////////////////////////////////////
//                                                         
// Bitmap.cc
// ---------
// Bitmap management 
//                                               
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2005 Bjoern Lemke                        
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: Bitmap
// 
// Description:
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////

#include <stdlib.h>

// INCLUDES
#include "Bitmap.h"
#include "Exception.h"

#define BIT_PER_BYTE 8

Bitmap::Bitmap(int size)
{    

    _size = size;
    _pBM = (char*)malloc(size / (BIT_PER_BYTE * sizeof(char)) + 1);
     
     if ( _pBM == 0 ) 
     {
	 throw Exception(EXLOC, "malloc system error"); 
     }

     for (int i = 0; i < (size / (BIT_PER_BYTE * sizeof(char)) + 1); i++)
     {
	 _pBM[i]=0;
     }
}


Bitmap::~Bitmap()
{
    free(_pBM);
}

void Bitmap::setValue(int pos, bool val)
{

    if ( pos < _size )
    {
	unsigned bmid = pos / ( BIT_PER_BYTE * sizeof(char));
	char bmoffset = pos % ( BIT_PER_BYTE * sizeof(char));
	
	cout << "bmid is " << bmid << endl;
	cout << "offset is " << (int)bmoffset << endl;
	
	char f = ~(~0 << 1);
	f = f << bmoffset;
	
	_pBM[bmid] = _pBM[bmid] | f;
    }
    else
    {
	throw Exception(EXLOC, "bitmap position exceeded"); 
    }
}


bool Bitmap::getValue(int pos) const
{
    if ( pos < _size )
    {
	unsigned bmid = pos / ( BIT_PER_BYTE * sizeof(char));
	char bmoffset = pos % ( BIT_PER_BYTE * sizeof(char));
	
	char checkbit = _pBM[bmid] >> bmoffset & ~(~0 << 1);    
	
	if (  checkbit )
	{
	    return true;
	}
	return false;
    }
    throw Exception(EXLOC, "bitmap position exceeded"); 

}


Chain Bitmap::toChain() const
{

    Chain s;
    for (int i = 0; i < _size; i++)
    {
	
	if ( getValue(i) )
	{
	    s += "1";
	}
	else
	{
	    s += "0";
	}
    }   
    return s;
}


Bitmap& Bitmap::operator = ( const Bitmap& bm)
{

    free(_pBM);

    _size = bm._size;
    _pBM = (char*)malloc(_size / (BIT_PER_BYTE * sizeof(char)) + 1);
     
     if ( _pBM == 0 ) 
     {
	 throw Exception(EXLOC, "malloc system error"); 
     }

     for (int i = 0; i < (_size / (BIT_PER_BYTE * sizeof(char)) + 1); i++)
     {
	 _pBM[i]=bm._pBM[i];
     }

     return (*this);
    
}

bool Bitmap::operator == ( const Bitmap& bm) const
{

    if ( _size != bm._size )
	return false;

    for (int i = 0; i < (_size / (BIT_PER_BYTE * sizeof(char)) + 1); i++)
    {
	if ( _pBM[i] != bm._pBM[i] )
	    return false;
	    
    }

    return true;
}



