#ifndef _CHAIN_H_INCLUDED_
#define _CHAIN_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// Chain.h
// --------
// string class definition
//                                                         
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2005 Bjoern Lemke                        
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// INTERFACE MODULE
//
// Class: Chain
// 
// Description: all base operations on strings
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////

// INCLUDES

#include <iostream>
using namespace std;

class Chain {

public:
  
  Chain();
  Chain(const char* s);
  Chain(const char* s, int len);
  Chain(const Chain& str);
  Chain(const char c);
  Chain(unsigned long l);
  Chain(long l);
  Chain(int i);
  Chain(float l);
  Chain(double l);
  
  ~Chain();

  void setData(char* s);

  // member methods
  unsigned long length() const;
  Chain toUpper() const;
  Chain toLower() const;

  int asInteger() const;
  long asLong() const;
  bool asBool() const;
  bool isNum() const;
  bool isDec() const;
  float asFloat() const;
  double asDouble() const;
  short asShort() const;
  char asChar() const;

  Chain cutTrailing(const Chain& str) const;
  Chain truncLeft(const Chain& str) const;
  Chain truncRight(const Chain& str) const;

  bool posStr(const Chain& s1, int& result, int start = 0, int occur = 1) const;
    
  bool replace(const Chain& s1, const Chain& s2, Chain& result) const;
  int replaceAll(const Chain& s1, const Chain& s2, Chain& result) const;

  void setChar(int i, char c);

  int getHashPos(int hashSize) const;

  // member operators

  char operator[] (int i) const;
  Chain subChain(int l, int u) const;
  operator char* () const;
  Chain& operator = ( const Chain& str);
  Chain& operator += ( const Chain& str);
  bool operator == ( const Chain& str) const;
  bool operator != ( const Chain& str) const;
  bool operator < ( const Chain& str) const;
  bool operator > ( const Chain& str) const;
  bool operator <= ( const Chain& str) const;
  bool operator >= ( const Chain& str) const;

  // friend operators
  friend Chain operator + ( const Chain& str1, const Chain& str2);
  friend ostream& operator << (ostream& s, const  Chain& str);  
  friend istream& operator >> (istream& s, Chain &str);

private:

  bool matchAtPos(const Chain& s1, int pos) const;

  char *_buf;
  unsigned long _len;

};



#endif



