///////////////////////////////////////////////////////////////////////////////
//                                                         
// CommandExecuter.cc
// ------------------
// Executing commands via process forking 
//                                               
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2007 Bjoern Lemke                        
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: CommandExecuter
// 
// Description: execution of 
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////

// SYSTEM INCLUDES
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/wait.h>


// BASE INCLUDES
#include "Exception.h"

#include "CommandExecuter.h"
#include "Tokenizer.h"

// DEFINES
#define BUFLEN 100
#define SHELLPATH "/bin/bash"
#define SHELL "bash"

CommandExecuter::CommandExecuter() : SigHandler()
{    
}

CommandExecuter::~CommandExecuter()  
{
}


int CommandExecuter::execute(const Chain& command, int timeout)
{

    alarm(0);
    alarm(timeout);

    install(SIGALRM);
    
    int pid;
    if ((pid = fork ()) > 0)
    {
	_childPid = pid;

	int status;
	int pid = wait(&status);
	// int pid = waitpid(pid, &status, WUNTRACED);

	alarm(0);
	if ( _childPid != 0 )
	    return WEXITSTATUS(status);
	else
	{
	    Chain msg = "Timeout occured for command <" +  command + ">";
	    throw Exception(EXLOC, msg);
	}
    }
    else if (pid == 0) 
    { 
	char* argArray[4]; 
	argArray[0]=(char*)SHELL;
	argArray[1]=(char*)"-c";
	argArray[2]=(char*)command;
	argArray[3]=0;


	int result;
	if ( (result = execv(SHELLPATH, argArray)) == -1 ) 
	{
	    free (argArray);
	    Chain msg = "Execution error : " + Chain(strerror(errno));
	    throw Exception(EXLOC, msg);
	}
	
	exit (0);
    }
    else
    {
	Chain msg = "Cannot execute command <" +  command + ">";
	throw Exception(EXLOC, msg);
    }
}

void CommandExecuter::sigCatch(int sig)
{	    
    kill(_childPid, SIGTERM);
    _childPid = 0;
}; 

