#ifndef _DATETIME_H_INCLUDED_
#define _DATETIME_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// Datetime.h
// ----------
// implementation of a datetime data type
//                                                         
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2007 Bjoern Lemke                        
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// INTERFACE MODULE
//
// Class: Datetime
// 
// Description: datetime utility class
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////

#include <time.h>
#include "Chain.h"


#ifdef HAVE_MINGW32
extern "C" char* strptime(const char* s,const char* format, struct tm* tm);
#endif


class Datetime {

public:
  
  Datetime();
  Datetime(const Chain& v);
  Datetime(const Chain& v, const Chain& format);
  Datetime(int i);
  
  ~Datetime();

  Chain asChain() const;
  Chain asChain(const Chain& format) const;
  int asInt() const;

  void add(int seconds);
  void sub(int seconds);

  // member operators

  Datetime& operator = ( const Datetime& d);
  bool operator == ( const Datetime& str) const;
  bool operator != ( const Datetime& str) const;
  bool operator < ( const Datetime& str) const;
  bool operator > ( const Datetime& str) const;
  bool operator <= ( const Datetime& str) const;
  bool operator >= ( const Datetime& str) const;

  // friend operators
  friend ostream& operator << (ostream& s, const  Datetime& str);
  

private:

  time_t _t;

};



#endif



