#ifndef _FILE_H_INCLUDED_
#define _FILE_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// File.h
// ------
// file class definition
//                                                         
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2005 Bjoern Lemke
//
// Design and Implementation by Bjoern Lemke               
//                                                         
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// INTERFACE MODULE
//
// Class: File
//
// Description: all operations on files 
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////


// INCLUDES

#include "Chain.h"

#define MAXLINELEN 1024

class File {
 
 public:
  
     enum Mode { CLOSED, READ, WRITE, READWRITE, APPEND };  

     File();
     File(char* s);
     File(const Chain& str);
     ~File();

     bool exists();

     void open(Mode m);
     void close();
     
     void writeByte(char *buf, unsigned long buflen);
     unsigned long readByte(char *buf, unsigned long buflen);
     
     void writeChain(const Chain& str);
     int readLine(Chain& line, long maxLen = MAXLINELEN);
     
     void seek(unsigned long pos);

     void trunc(unsigned long size);
     
     void remove();

     int Size() const;

     const Chain& getFileName() const;
     Chain getShortName() const;

     File& operator =  ( File& f);
     File& operator += ( File& f);
     
     File& operator << ( const Chain& str);
     File& operator << ( unsigned long l);
     File& operator << ( int i);
     File& operator << ( char c);
     
 private:
     
     int _fd;
     Mode _mode;
     Chain _filename;
  
  
};

#endif

