///////////////////////////////////////////////////////////////////////////////
//                                                         
// GetLongOpt.cc
// -------------
// GetLongOpt class implementation
//                                               
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2009 Bjoern Lemke                        
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: GetLongOpt
// 
// Description: Handling command line arguments
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////


// SYSTEM INCLUDES
#include <string.h>

// BASE INCLUDES
#include "GetLongOpt.h"

#include "Chain.h"
#include "Exception.h"
#include "Tokenizer.h"

GetLongOpt::GetLongOpt(int argc, char** argv)
{
    _argc = argc;
    _argv = argv;
    _isParsed = false;
}

GetLongOpt::~GetLongOpt()
{
}

void GetLongOpt::addOpt(const Chain& opt)
{
    _optHash.Insert(OptHash(opt));
}

void GetLongOpt::addOpt(const Chain& opt, const Chain& defVal)
{
    _optHash.Insert(OptHash(opt, defVal));
}

Chain GetLongOpt::getOptValue(const Chain& opt)
{
    if ( _isParsed == false )
    {
	parseOpt();
    }

    OptHash *pOH =  _optHash.Find(OptHash(opt));
    if ( pOH )
	return pOH->getValue();
    
    Chain msg = Chain("Invalid option ") + opt;		    
    throw Exception(EXLOC, msg);
}

bool GetLongOpt::isSet(const Chain& opt)
{
    if ( _isParsed == false )
    {
	parseOpt();
    }

    OptHash *pOH = _optHash.Find(OptHash(opt));
    if ( pOH )
	return pOH->isSet();
    
    Chain msg = Chain("Invalid option ") + opt;		    
    throw Exception(EXLOC, msg);
    
}
void GetLongOpt::parseOpt()
{
    int optPos = 1;
    while (optPos < _argc )
    {

	Tokenizer tok(Chain(_argv[optPos]), Chain("="));
	Chain opt;
	if ( tok.nextToken(opt) )
	{
	    opt = opt.cutTrailing("-");
	    
	    OptHash *pOH = _optHash.Find(OptHash(opt));
	    if ( pOH == 0)
	    {
		Chain msg = Chain("Invalid option ") + opt;		    
		throw Exception(EXLOC, msg);
	    }

	    pOH->set();
		
	    Chain val;
	    if ( tok.nextToken(val) )
	    {		
		pOH->setValue(val);
	    }
	}
	optPos++;
    }

    _isParsed = true;
}
