///////////////////////////////////////////////////////////////////////////////
//                                                         
// GetOpt.cc
// ---------
// GetOpt class implementation
//                                               
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2005 Bjoern Lemke                        
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: GetOpt
// 
// Description: Handling command line arguments
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////


// SYSTEM INCLUDES
#include <string.h>

// BASE INCLUDES
#include "Chain.h"
#include "Exception.h"
#include "GetOpt.h"

GetOpt::GetOpt(int argc, char** argv, char* pF)
{
    _argc = argc;
    _argv = argv;
    _pF = pF;
}

GetOpt::~GetOpt()
{
}

int GetOpt::firstOpt()
{
    _optPos = 1;
    return parseOpt();
}

int GetOpt::nextOpt()
{
    _optPos++;
    return parseOpt();
}

int GetOpt::parseOpt()
{
    if (_optPos < _argc )
    {
	
	if (strlen(_argv[_optPos]) == 2 && _argv[_optPos][0] == '-')
	{
	    int c = _argv[_optPos][1];
	    if ( isValidOption(c) )
	    {
		return c;    
	    }
	    else
	    {
		Chain msg = Chain("Invalid option -") + Chain((char)c);
		throw Exception(EXLOC, msg);
	    }
	}
	else
	{
	    Chain msg = Chain("Invalid option ") + Chain(_argv[_optPos]);
	    throw Exception(EXLOC, msg);
	}
    }
    else
    {
	return 0;
    }
}

bool GetOpt::isValidOption(int c)
{
    char *pF = _pF;
    while (*pF)
    {
	if (*pF == c)
	{
	    pF++;
	    if (*pF != 0)
	    {
		if (*pF == ':')
		{
		    _optPos++;
		    _optArg = _argv[_optPos];
		}
	    }
	    return true;
	}
	else
	{
	    pF++;
	}   
    }
    return false;
    
}

char* GetOpt::getArg()
{
    return _optArg;
}
