///////////////////////////////////////////////////////////////////////////////
//                                                         
// Logger.cc
// ---------
// Managing log messages
//                                               
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2005 Bjoern Lemke                        
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: Logger
// 
// Description: Managing module notices and debug messages
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////


// INCLUDES
#include "Datetime.h"
#include "Logger.h"

// DEFINES

Logger::Logger()
{
    _pF = 0;
}

Logger::Logger(const Chain& logFile)
{
    _pF = new File(logFile);
    if (_pF)
    {
	_pF->open(File::APPEND);
    }
    _logLevel = Logger::NOTICE;
    _modId = 0;
}

Logger::~Logger()
{
    if (_pF)
    {
	delete _pF;
    }
}

void Logger::logModule(unsigned long modId, const Chain& module, Logger::LogLevel level)
{
    if ( modId > 0 )
    {
	ModEntry *pME = _modSet.Find(ModEntry(modId));
	if ( pME )
	{
	    pME->setModule(module);
	    pME->setLevel(level);
	}
	else
	{
	    _modSet.Insert(ModEntry(modId, module, level));
	}
    }
}
	
void Logger::log(unsigned long modId, Logger::LogLevel level, const Chain& msg)
{
    ModEntry *pME = _modSet.Find(ModEntry(modId));

    if ( pME && _pF )
    {
	// cout << "Logging <" << msg << " on level " << level << " MsgLevel = " << pME->getLogLevel() << endl;
	if ( level <= pME->getLogLevel()   )
	{
	    Chain cat;
	    
	    switch (level)
	    {
	    case NOTICE:
		cat = Chain("NOTICE");
		break;
	    case LOGERR:
		cat = Chain("ERROR");
		break;
	    case DEBUG:
		cat = Chain("DEBUG");
		break;
	    }

	    Datetime dt;
	    
	    _pF->writeChain(dt.asChain() + Chain(" [") + pME->getModule() + Chain("] ") + cat + Chain(" : ") + msg + Chain("\n"));
	}
    }
}

Logger& Logger::operator << ( unsigned long modId)
{
    ModEntry *pME = _modSet.Find(ModEntry(modId));
    
    if ( pME )
    {
	_modId = modId;
    }
    else
    {
	_modId = 0;
    }
    return (*this);
}


Logger& Logger::operator << ( const Logger::LogLevel& level)
{

    ModEntry *pME = _modSet.Find(ModEntry(_modId));

    if ( pME && _pF )
    {
	_msgLevel = level;
	_logLevel = pME->getLogLevel();
	if ( level <= _logLevel   )
	{
	    Chain cat;
	    Datetime dt;
	    switch (level)
	    {
	    case NOTICE:
		cat = Chain("NOTICE");
		break;
	    case LOGERR:
		cat = Chain("ERROR");
		break;
	    case DEBUG:
		cat = Chain("DEBUG");
		break;
	    }
	    _pF->writeChain(dt.asChain() + + Chain(":") + pME->getModule() + Chain(":") + cat + Chain(":"));
	}
    }
    return (*this);
}
    
Logger& Logger::operator << ( const Chain& str)
{
    if ( _pF && _msgLevel <= _logLevel && _modId > 0 )
	_pF->writeChain(str);
    return (*this);
}


Logger& Logger::operator << ( const char* str)
{
    if ( _pF && _msgLevel <= _logLevel && _modId > 0 )
    {
	Chain s(str);
	_pF->writeChain(s);
    }
    return (*this);
}


/*
Logger& Logger::operator << ( char c)
{
    if ( _pF && _msgLevel <= _logLevel)
    {
	char buf[2];
	buf[0] = c;
	buf[1] = 0;
	
	_pF->writeChain(buf);
    }
    return (*this);
}
*/




