///////////////////////////////////////////////////////////////////////////////
//                                                         
// Matcher.cc
// ----------
// Regular Expression handling
//                                               
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2007 Bjoern Lemke                        
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: Matcher
// 
// Description:
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////


// INCLUDES

#include "Exception.h"
#include "Chain.h"
#include "Matcher.h"

Matcher::Matcher(const Chain& expr)
{    
    _expr = expr;
    _isPrepared = false;
}

Matcher::~Matcher()
{

}

void Matcher::prepare()
{

    if ( (char*)_expr != 0 )
    {
	if ( regcomp(&_re, (char*)_expr, 0) != 0 )
	{
	    throw Exception(EXLOC, "regcomp error"); 
	}
    }
    else
    {
	Chain msg = Chain("Invalid regular expression <") + _expr + Chain(">"); 
	throw Exception(EXLOC, msg); 
    }
    _isPrepared = true;    
}

bool Matcher::match(const Chain& str)
{
    if ( _isPrepared )
    {
	if ( (char*)str != 0  )
	    if (::regexec(&_re, (char*)str, 0, NULL, 0) == 0)
		return true;
	return false;
    }
    throw Exception(EXLOC, "Matcher not prepared"); 
}




