///////////////////////////////////////////////////////////////////////////////
//                                                         
// NanoTimer.cc
// ------------
// NanoTimer implementation
//                                               
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2005 Bjoern Lemke                        
//                                                         
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: NanoTimer
// 
// Description: NanoTimer implementation
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////



#include "Exception.h"
#include "NanoTimer.h"

#ifdef HAVE_MINGW32
#include <windows.h>
#else
#include <sys/time.h>
#endif

#ifdef HAVE_MACH_TIMEBASE_INFO
#include <mach/mach_time.h>
static mach_timebase_info_data_t s_timebase_info;
#endif

NanoTimer::NanoTimer()
{
    _start = 0;
    _end = 0;
    _sum = 0;
    
#ifdef HAVE_MINGW32
    if (!QueryPerformanceFrequency((LARGE_INTEGER*)&_freq))
	throw Exception(EXLOC, "nano timer not available");
#endif

#if HAVE_MACH_TIMEBASE_INFO
    if (s_timebase_info.denom == 0) 
    {
	(void) mach_timebase_info(&s_timebase_info);
    }
#endif

}

NanoTimer::~NanoTimer()
{
}

void NanoTimer::start()
{

#ifdef HAVE_MINGW32
    QueryPerformanceCounter((LARGE_INTEGER*)&_start);
#else
    
#if HAVE_GETHRTIME
    _start = gethrtime();
#elif HAVE_CLOCK_GETTIME
    struct timespec tp;
    clock_gettime(CLOCK_REALTIME, &tp);
    _start = tp.tv_sec * 1000000000 + tp.tv_nsec;
#elif HAVE_MACH_TIMEBASE_INFO
    _start = mach_absolute_time();
#else    
    _start = 0;
#endif

#endif /* HAVE_MINGW32 */
    
}

void NanoTimer::stop()
{
#ifdef HAVE_MINGW32
    QueryPerformanceCounter((LARGE_INTEGER*)&_end);
    _sum = _sum + ( _end - _start / ( ( 1000000000 / _freq ) + 1 ));
#else

#if HAVE_GETHRTIME
    _end = gethrtime();
#elif HAVE_CLOCK_GETTIME
    struct timespec tp;
    clock_gettime(CLOCK_REALTIME, &tp);
    _end = tp.tv_sec * 1000000000 + tp.tv_nsec;
#elif HAVE_MACH_TIMEBASE_INFO
    _end = mach_absolute_time();
#else
    _end = 0;
#endif
    
    _sum = _sum + ( _end - _start );

#endif


}

void NanoTimer::reset()
{    
    _start = 0;
    _end = 0;
    _sum = 0;
}

long NanoTimer::getSum() const
{
    return _sum;
}

