///////////////////////////////////////////////////////////////////////////////
//                                                         
// Semaphore.cc
// ------------
// semaphore memory class implementation
//                                               
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2005 Bjoern Lemke                        
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: Semaphore
// 
// Description: all operations on system semaphore resources
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////


// BASE INCLUDES
#include "Exception.h"
#include "Semaphore.h"


#ifdef HAVE_MINGW32

#else

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>

#endif


#if ( ! HAVE_UNION_SEMUN )
union semun {
    int val;                    /* value for SETVAL */
    struct semid_ds *buf;       /* buffer for IPC_STAT, IPC_SET */
    unsigned short int *array;  /* array for GETALL, SETALL */
    struct seminfo *__buf;      /* buffer for IPC_INFO */
};
#endif

Semaphore::Semaphore(int key)
{

#ifdef HAVE_MINGW32
#else

    if ( (_id = semget(key, 1, IPC_CREAT | 0000777)) == -1)
    {
	throw Exception(EXLOC, "semget system error");
    }
#endif   
}

Semaphore::~Semaphore()
{
}

void Semaphore::init()
{
#ifdef HAVE_MINGW32
#else
    union semun arg; 
    
    arg.val=1;
    
    if ( semctl(_id, 0, SETVAL, arg) == -1)
    {
	// perror("semctl");
	throw Exception(EXLOC, "semctl system error");
    }
#endif
}

void Semaphore::destroy()
{
#ifdef HAVE_MINGW32
#else

    if ( semctl(_id, 0, IPC_RMID)  == -1)
    {
	throw Exception(EXLOC, "semctl system error");
    }
#endif
}

int Semaphore::getValue()
{
#ifdef HAVE_MINGW32
    return 0;
#else
    return semctl(_id, 0, GETVAL);
#endif
}

void Semaphore::P()
{
#ifdef HAVE_MINGW32
#else
    struct sembuf op;

    op.sem_num = 0;
    op.sem_op  = -1;
    op.sem_flg = 0;
    
 
    // cout << "P(" << _id << ")" << endl;

    if ( semop(_id, &op, 1) == -1)
    {
	// perror("semop:");
	throw Exception(EXLOC, "semop system error");
    }
#endif
}

void Semaphore::V()
{
#ifdef HAVE_MINGW32
#else
    struct sembuf op;
    
    op.sem_num = 0;
    op.sem_op  = 1;
    op.sem_flg = 0;
    
    // cout << "V(" << _id << ")" << endl;

    if ( semop(_id, &op, 1) == -1)
    {
	throw Exception(EXLOC, "semop system error");
    }
#endif
}

