///////////////////////////////////////////////////////////////////////////////
//                                                         
// SharedMemory.cc
// ---------------
// shared memory class implementation
//                                               
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2005 Bjoern Lemke                        
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: SharedMemory 
// 
// Description: all operations on system shared memory resources
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////


#include "Exception.h"
#include "SharedMemory.h"

#ifdef HAVE_MINGW32
#else
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <iostream>
#endif



SharedMemory::SharedMemory(int key)
{
     _key = key;
     _id = 0;
}

SharedMemory::~SharedMemory()
{
}

void SharedMemory::create(unsigned long size, int shmflag)
{

#ifdef HAVE_MINGW32
#else

    if (_id == 0)
    {
	if ( (_id = shmget(_key, size,  shmflag | IPC_CREAT)) == -1)
	{
	    throw Exception(EXLOC, "shmget system error");
	}
    }

    if ( (_addr = shmat(_id, (void*) 0,  shmflag)) == (void*)-1)
    {
	throw Exception(EXLOC, "shmat system error");
    }
#endif     
}

void SharedMemory::attach(int shmflag)
{

#ifdef HAVE_MINGW32
#else

    if (_id == 0)
    {
	if ( (_id = shmget(_key, 0,  shmflag)) == -1)
	{
	    throw Exception(EXLOC, "shmget system error");
	}
    }
    if ( (_addr = shmat(_id, (void*) 0,  shmflag)) == (void*)-1)
    {
	throw Exception(EXLOC, "shmat system error");
    }
#endif
}


void SharedMemory::destroy()
{

#ifdef HAVE_MINGW32
#else

    if (_id == 0)
    {
	if ( (_id = shmget(_key, 0,  0)) == -1)
	{
	    throw Exception(EXLOC, "shmget system error");
	}
    }
    if ( shmctl(_id, IPC_RMID,(struct shmid_ds*)0) == -1)
    {
	throw Exception(EXLOC, "shmctl system error");
    }
#endif    
}

void SharedMemory::detach()
{

#ifdef HAVE_MINGW32
#else

    if ( shmdt((char*)_addr) == -1)
     {
	 throw Exception(EXLOC,  "shmdt system error");
     }
#endif
}

int SharedMemory::getId()
{
     return _id;
}

void* SharedMemory::getAddr()
{
     return _addr;
}

void SharedMemory::printInfo()
{

#ifdef HAVE_MINGW32
#else


    struct shmid_ds ds;
    if ( shmctl(_id, IPC_STAT,&ds) == -1)
    {
	throw Exception(EXLOC, "shmctl system error");
    }

    cout << "Segment Size: " << ds.shm_segsz << " bytes"  <<endl;
    cout << "Creator Pid: " << ds.shm_cpid << endl;
    cout << "NumAttach: " << ds.shm_nattch << endl;
#endif  
}

