///////////////////////////////////////////////////////////////////////////////
//                                                         
// Sleeper.cc
// ----------
// Sleeper class implementation
//                                               
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2007 Bjoern Lemke                        
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: Sleeper
// 
// Description: all operations on network resources
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////

#ifndef _REENTRANT
#define _REENTRANT    /* basic 3-lines for threads */
#endif


#include "Sleeper.h"

#ifdef HAVE_MINGW32
#include <windows.h>
#endif

// SYSTEM INCLUDES
#include <time.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include "Exception.h"
#include "Chain.h"


Sleeper::Sleeper()
{
}

Sleeper::~Sleeper()
{
}

void Sleeper::secSleep(int n) const
{
#ifdef HAVE_MINGW32
    Sleep(n * 1000);
#else
    ::sleep(n);
#endif
}

void Sleeper::milliSleep(int n) const
{
#ifdef HAVE_MINGW32
    Sleep(n);
#else
    struct timespec delay;	
    delay.tv_sec = 0; 
    delay.tv_nsec = n * 1000000;
    if ( nanosleep(&delay, NULL) == -1 )
    {
	Chain msg = Chain("nanosleep system error : ") + Chain(strerror(errno));
	throw Exception(EXLOC, msg);
    }
#endif
}

void Sleeper::microSleep(int n) const
{
#ifdef HAVE_MINGW32
    Sleep(n / 1000);
#else
    struct timespec delay;	
    delay.tv_sec = 0; 
    delay.tv_nsec = n * 1000;
    if ( nanosleep(&delay, NULL) == -1 )
    {
	Chain msg = Chain("nanosleep system error : ") + Chain(strerror(errno));
	throw Exception(EXLOC, msg);
    }
#endif
}

void Sleeper::nanoSleep(int n) const
{
#ifdef HAVE_MINGW32
    Sleep(n / 1000000);
#else

    struct timespec delay;	
    delay.tv_sec = 0; 
    delay.tv_nsec = n;
    if ( nanosleep(&delay, NULL) == -1 )
    {
	Chain msg = Chain("nanosleep system error : ") + Chain(strerror(errno));
	throw Exception(EXLOC, msg);
    }
#endif
}




