#ifndef _THREADLOCK_H_INCLUDED_
#define _THREADLOCK_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// ThreadLock.h
// ------------
// ThreadLock class interface definition
//                                                         
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2005 Bjoern Lemke                        
//                                                         
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// INTERFACE MODULE
//
// Class: ThreadLock
// 
// Description: all operations on POSIX thread locks
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////

#ifndef _REENTRANT
#define _REENTRANT    /* basic 3-lines for threads */
#endif

#ifdef HAVE_MINGW32
#include <windows.h>
#else
#include <pthread.h>
#endif

#include "Chain.h"


#define LOCKDELAY 100

class ThreadLock 
{
    
public:

    ThreadLock(const Chain& id = "");
    ~ThreadLock();

    const Chain& getId() const;
    void setId(const Chain& id);

    void init(int lockDelay = LOCKDELAY, bool doStat = true);
    void readLock();
    void writeLock();

    void readLock(int timeout);
    void writeLock(int timeout);
    
    void unlock();

    long numLockTry();
    
    long numReadLock();
    long numWriteLock();
    long sumReadDelay();
    long sumWriteDelay();

private:

    Chain _id;
    int _lockDelay;

#ifdef HAVE_MINGW32
    CRITICAL_SECTION _rwlock;
#else
    pthread_rwlock_t _rwlock;    
    pthread_rwlock_t _statlock;    
#endif

    bool _doStat;

    long _readLockCount;
    long _writeLockCount;
    long _readDelay;
    long _writeDelay;

    long _numLockTry;


};

#endif

